==> web-10.0.0.tgz: package/lib/scroll/ScrollBox.js <==
import * as React from "react";
import { fnUtil } from "../util/jsUtil";
import { tween } from "../animate/tween";
class ScrollBox extends React.PureComponent {
    constructor(props) {
        super(props);
        this.moveHBar = React.createRef();
        this.moveWBar = React.createRef();
        this.scrollMain = React.createRef();
        this.state = {
            showHBar: !!this.props.keepBarShow,
            showWBar: !!this.props.keepBarShow,
            moveBarH: 0,
            moveBarW: 0,
        };
        this.scrollBoxH = 0;
        this.scrollBoxW = 0;
        this.scrollMainH = 0;
        this.scrollMainW = 0;
        this.timerH = 0;
        this.timerW = 0;
        this.upTimer = 0;
        this.moveStartTop = null;
        this.scrollToTop = (top) => {
            const { moveBarH } = this.state;
            const scrollMain = this.scrollMain.current;
            let h = top;
            const maxH = this.scrollMainH - this.scrollBoxH;
            if (maxH <= 0) {
                return;
            }
            h = h < 0 ? 0 : h > maxH ? maxH : h;
            const factor = (this.scrollBoxH - moveBarH) / maxH;
            scrollMain.style.top = `${-h}px`;
            const val = Math.floor(h * factor);
            this.moveHBar.current.style.top = `${val}px`;
        };
        this.scrollToLeft = (left) => {
            const { moveBarW } = this.state;
            const scrollMain = this.scrollMain.current;
            let w = left;
            const maxW = this.scrollMainW - this.scrollBoxW;
            if (maxW <= 0) {
                return;
            }
            w = w < 0 ? 0 : w > maxW ? maxW : w;
            const factor = (this.scrollBoxW - moveBarW) / maxW;
            scrollMain.style.left = `${-w}px`;
            const val = Math.floor(w * factor);
            this.moveWBar.current.style.left = `${val}px`;
        };
        this.barHMove = (e) => {
            const { moveBarH } = this.state;
            const dom = this.moveHBar.current;
            const scrollMain = this.scrollMain.current;
            const startH = +dom.dataset.top;
            const moveDistance = e.clientY - +dom.dataset.y;
            const maxH = this.scrollBoxH - moveBarH;
            let h = startH + moveDistance;
            h = h < 0 ? 0 : h > maxH ? maxH : h;
            const factor = (this.scrollMainH - this.scrollBoxH) / maxH;
            dom.style.top = `${h}px`;
            const top = Math.ceil(-h * factor);
            scrollMain.style.top = `${top}px`;
            const scrollCallback = this.props.scrollCallback;
            scrollCallback && scrollCallback(top, true);
        };
        this.barWMove = (e) => {
            const { moveBarW } = this.state;
            const dom = this.moveWBar.current;
            const scrollMain = this.scrollMain.current;
            const startW = +dom.dataset.left;
            const moveDistance = e.clientX - +dom.dataset.x;
            const maxW = this.scrollBoxW - moveBarW;
            let w = startW + moveDistance;
            w = w < 0 ? 0 : w > maxW ? maxW : w;
            const factor = (this.scrollMainW - this.scrollBoxW) / maxW;
            dom.style.left = `${w}px`;
            const left = Math.ceil(-w * factor);
            scrollMain.style.left = `${left}px`;
            const scrollCallback = this.props.scrollCallback;
            scrollCallback && scrollCallback(left, false);
        };
        this.barHMoveEven = fnUtil.throttle(this.barHMove, 100);
        this.barWMoveEven = fnUtil.throttle(this.barWMove, 100);
        this.upDateInit = (defaultH) => {
            this.updateVertical(defaultH);
            this.updatehorizontal();
        };
        this.barHClick = (e) => {
            e.currentTarget.dataset.y = `${e.clientY}`;
            const top = parseInt(e.currentTarget.style.top || "0", 0);
            e.currentTarget.dataset.top = `${top}`;
            e.currentTarget.parentElement.parentElement.addEventListener("mousemove", this.barHMoveEven);
            this.scrollMain.current.classList.add("no-selected");
        };
        this.barWClick = (e) => {
            e.currentTarget.dataset.x = `${e.clientX}`;
            const left = parseInt(e.currentTarget.style.left || "0", 0);
            e.currentTarget.dataset.left = `${left}`;
            e.currentTarget.parentElement.parentElement.addEventListener("mousemove", this.barWMoveEven);
            this.scrollMain.current.classList.add("no-selected");
        };
        this.cancelBarMove = () => {
            this.moveHBar.current.parentElement.parentElement.removeEventListener("mousemove", this.barHMoveEven);
            this.moveWBar.current.parentElement.parentElement.removeEventListener("mousemove", this.barWMoveEven);
            this.scrollMain.current.classList.remove("no-selected");
        };
        this.mouseOver = (e) => {
            const dom = e.currentTarget;
            if (!dom.firstElementChild) {
                return;
            }
            if (this.state.showWBar) {
                const curScrollBoxW = this.scrollMain.current.parentElement.clientWidth;
                if (curScrollBoxW !== this.scrollBoxW) {
                    this.updatehorizontal();
                }
            }
            if (this.scrollMainH > this.scrollBoxH) {
                dom.onwheel = this.stopWhell;
                this.setState({
                    showHBar: true,
                });
                this.moveStartTop = parseInt(this.moveHBar.current.style.top || "0", 0);
            }
            else {
                dom.onwheel = null;
            }
        };
        this.mainMousewheel = (e) => {
            this.cancelBarMove();
            const { showHBar, moveBarH } = this.state;
            if (!showHBar) {
                return;
            }
            const dom = e.currentTarget;
            const scrollMain = this.scrollMain.current;
            const startH = parseInt(scrollMain.style.top || "0", 0);
            if (this.moveStartTop === null) {
                dom.onwheel = this.stopWhell;
            }
            else {
                if (e.deltaY < 0) {
                    if (this.moveStartTop < 5) {
                        dom.onwheel = null;
                    }
                    else {
                        dom.onwheel = this.stopWhell;
                        this.moveStartTop = null;
                    }
                }
                else {
                    if (this.moveStartTop > this.scrollBoxH - moveBarH - 5) {
                        dom.onwheel = null;
                    }
                    else {
                        dom.onwheel = this.stopWhell;
                        this.moveStartTop = null;
                    }
                }
            }
            let step = Math.sqrt(this.scrollMainH / this.scrollBoxH) * 15;
            step = Math.floor(step);
            step = e.deltaY < 0 ? step : -step;
            let h = -(startH + step);
            const maxH = this.scrollMainH - this.scrollBoxH;
            h = h < 0 ? 0 : h > maxH ? maxH : h;
            const factor = (this.scrollBoxH - moveBarH) / maxH;
            scrollMain.style.top = `${-h}px`;
            this.moveHBar.current.style.top = `${Math.floor(h * factor)}px`;
            const scrollCallback = this.props.scrollCallback;
            scrollCallback && scrollCallback(-h, true);
        };
        this.locateBarH = (e) => {
            const { moveBarH } = this.state;
            let h = e.nativeEvent.offsetY;
            const dom = this.moveHBar.current;
            const maxH = this.scrollBoxH - moveBarH;
            const factor = (this.scrollMainH - this.scrollBoxH) / maxH;
            const scrollMain = this.scrollMain.current;
            h = h < 0 ? 0 : h > maxH ? maxH : h;
            dom.style.top = `${h}px`;
            const top = Math.ceil(-h * factor);
            scrollMain.style.top = `${top}px`;
            const scrollCallback = this.props.scrollCallback;
            scrollCallback && scrollCallback(top, true);
        };
        this.locateBarW = (e) => {
            const { moveBarW } = this.state;
            let w = e.nativeEvent.offsetX;
            const dom = this.moveWBar.current;
            const maxW = this.scrollBoxW - moveBarW;
            const factor = (this.scrollMainW - this.scrollBoxW) / maxW;
            const scrollMain = this.scrollMain.current;
            w = w < 0 ? 0 : w > maxW ? maxW : w;
            dom.style.left = `${w}px`;
            const left = Math.ceil(-w * factor);
            scrollMain.style.left = `${left}px`;
            const scrollCallback = this.props.scrollCallback;
            scrollCallback && scrollCallback(left, false);
        };
        this.stopProp = (e) => {
            e.stopPropagation();
        };
        this.stopWhell = (e) => {
            e.preventDefault();
        };
        this.scrollMethods = {
            initScroll: this.upDateInit,
            scrollToTop: this.scrollToTop,
            scrollToLeft: this.scrollToLeft,
        };
        if (props.bindIntiScroll) {
            props.bindIntiScroll(this.scrollMethods);
        }
    }
    componentDidMount() {
        this.upDateInit(this.props.height);
        document.body.addEventListener("mouseup", this.cancelBarMove);
    }
    componentWillUnmount() {
        document.body.removeEventListener("mouseup", this.cancelBarMove);
        window.cancelAnimationFrame(this.timerH);
        window.cancelAnimationFrame(this.timerW);
        window.cancelAnimationFrame(this.upTimer);
        this.moveHBar.current.parentElement.parentElement.onwheel = null;
    }
    updateVertical(setH) {
        if (!this.moveHBar.current) {
            return;
        }
        const barInner = this.moveHBar.current;
        const scrollMain = this.scrollMain.current;
        const curScrollMainH = scrollMain.clientHeight;
        const curScrollBoxH = scrollMain.parentElement.clientHeight;
        if (!setH && curScrollMainH === this.scrollMainH && curScrollBoxH === this.scrollBoxH) {
            return;
        }
        this.scrollBoxH = setH ? setH : curScrollBoxH;
        this.scrollMainH = curScrollMainH;
        if (curScrollMainH <= this.scrollBoxH) {
            scrollMain.style.top = "0";
            this.setState({
                showHBar: false,
            });
            return;
        }
        let moveBarH = Math.ceil((this.scrollBoxH / curScrollMainH) * this.scrollBoxH);
        moveBarH = Math.max(16, moveBarH);
        const top = scrollMain.style.top || "0";
        let curScrollMainTop = -parseInt(top, 0);
        if (curScrollMainTop + this.scrollBoxH > curScrollMainH) {
            curScrollMainTop = curScrollMainH - this.scrollBoxH;
            scrollMain.style.top = `${-curScrollMainTop}px`;
        }
        const maxH = this.scrollMainH - this.scrollBoxH;
        const factor = (this.scrollBoxH - moveBarH) / maxH;
        barInner.style.top = `${Math.floor(factor * curScrollMainTop)}px`;
        this.animateScroll({
            start: this.state.moveBarH,
            end: moveBarH,
            isVertical: true
        }, () => {
            this.setState({
                moveBarH,
                showHBar: true,
            });
        });
    }
    updatehorizontal(setW) {
        if (!this.moveWBar.current) {
            return;
        }
        const barInner = this.moveWBar.current;
        const scrollMain = this.scrollMain.current;
        const curScrollMainW = scrollMain.clientWidth;
        const curScrollBoxW = scrollMain.parentElement.clientWidth;
        if (!setW && curScrollMainW === this.scrollMainW && curScrollBoxW === this.scrollBoxW) {
            return;
        }
        this.scrollBoxW = setW ? setW : curScrollBoxW;
        this.scrollMainW = curScrollMainW;
        if (curScrollMainW <= this.scrollBoxW) {
            scrollMain.style.left = "0";
            this.setState({
                showWBar: false,
            });
            return;
        }
        let moveBarW = Math.ceil((this.scrollBoxW / curScrollMainW) * this.scrollBoxW);
        moveBarW = Math.max(16, moveBarW);
        const left = scrollMain.style.left || "0";
        let curScrollMainLeft = -parseInt(left, 0);
        if (curScrollMainLeft + this.scrollBoxW > curScrollMainW) {
            curScrollMainLeft = curScrollMainW - this.scrollBoxW;
            scrollMain.style.left = `${-curScrollMainLeft}px`;
        }
        const maxW = this.scrollMainW - this.scrollBoxW;
        const factor = (this.scrollBoxW - moveBarW) / maxW;
        barInner.style.left = `${Math.floor(factor * curScrollMainLeft)}px`;
        this.animateScroll({
            start: this.state.moveBarW,
            end: moveBarW,
            isVertical: false
        }, () => {
            this.setState({
                moveBarW,
                showWBar: true,
            });
        });
    }
    animateScroll(obj, callback) {
        const { start, end, isVertical } = obj;
        if (isVertical) {
            window.cancelAnimationFrame(this.timerH);
            this.timerH = 0;
        }
        else {
            window.cancelAnimationFrame(this.timerW);
            this.timerW = 0;
        }
        let time = 0;
        const timeEnd = Math.floor(Math.sqrt(Math.abs(end - start)));
        const fn = isVertical ? () => {
            this.timerH = window.requestAnimationFrame(() => {
                const distance = String(tween.easeInCubic(time, start, end, timeEnd));
                this.moveHBar.current.style.height = distance + "px";
                if (time < timeEnd) {
                    fn();
                }
                else {
                    this.timerH = 0;
                    callback();
                }
                time++;
            });
        } : () => {
            this.timerW = window.requestAnimationFrame(() => {
                const distance = String(tween.easeInCubic(time, start, end, timeEnd));
                this.moveWBar.current.style.width = distance + "px";
                if (time < timeEnd) {
                    fn();
                }
                else {
                    this.timerW = 0;
                    callback();
                }
                time++;
            });
        };
        fn();
    }
    componentDidUpdate(prevProps) {
        if (prevProps.height !== this.props.height) {
            this.upDateInit(this.props.height);
        }
    }
    render() {
        const { className, height, children, scrollWidthAuto } = this.props;
        const { showHBar, moveBarH, moveBarW, showWBar } = this.state;
        const style = height ? { height: `${height}px` } : undefined;
        return (React.createElement("div", { className: `g-scrollBox ${className}`, style: style, onMouseEnter: this.mouseOver, onWheel: this.mainMousewheel },
            React.createElement("div", { style: { display: showHBar ? "block" : "none", }, className: "m-scrollBar m-scrollHBar", onClick: this.locateBarH },
                React.createElement("div", { ref: this.moveHBar, className: "m-moveBar m-moveHBar", style: { height: `${moveBarH}px`, top: "0", }, onClick: this.stopProp, onMouseDown: this.barHClick })),
            React.createElement("div", { className: "wrap-main" },
                React.createElement("div", { className: "m-scrollMain", style: { width: scrollWidthAuto ? undefined : "100%" }, ref: this.scrollMain }, children)),
            React.createElement("div", { style: { display: showWBar ? "block" : "none", }, className: "m-scrollBar m-scrollWBar", onClick: this.locateBarW },
                React.createElement("div", { ref: this.moveWBar, className: "m-moveBar m-moveWBar", style: { width: `${moveBarW}px`, left: "0", }, onClick: this.stopProp, onMouseDown: this.barWClick }))));
    }
}
ScrollBox.defaultProps = {
    className: "",
};
export default ScrollBox;

==> web-11.0.2.tgz: package/lib/scroll/ScrollBox.js <==
import * as React from "react";
import { fnUtil } from "../util/jsUtil";
import { tween } from "../animate/tween";
class ScrollBox extends React.PureComponent {
    constructor(props) {
        super(props);
        this.moveHBar = React.createRef();
        this.moveWBar = React.createRef();
        this.scrollMain = React.createRef();
        this.startY = null;
        this.state = {
            showHBar: !!this.props.keepBarShow,
            showWBar: !!this.props.keepBarShow,
            moveBarH: 0,
            moveBarW: 0,
        };
        this.scrollBoxH = 0;
        this.scrollBoxW = 0;
        this.scrollMainH = 0;
        this.scrollMainW = 0;
        this.timerH = 0;
        this.timerW = 0;
        this.upTimer = 0;
        this.moveStartTop = null;
        this.scrollToTop = (top) => {
            const { moveBarH } = this.state;
            const scrollMain = this.scrollMain.current;
            let h = top;
            const maxH = this.scrollMainH - this.scrollBoxH;
            if (maxH <= 0) {
                return;
            }
            h = h < 0 ? 0 : h > maxH ? maxH : h;
            const factor = (this.scrollBoxH - moveBarH) / maxH;
            scrollMain.style.top = `${-h}px`;
            const val = Math.floor(h * factor);
            this.moveHBar.current.style.top = `${val}px`;
        };
        this.scrollToLeft = (left) => {
            const { moveBarW } = this.state;
            const scrollMain = this.scrollMain.current;
            let w = left;
            const maxW = this.scrollMainW - this.scrollBoxW;
            if (maxW <= 0) {
                return;
            }
            w = w < 0 ? 0 : w > maxW ? maxW : w;
            const factor = (this.scrollBoxW - moveBarW) / maxW;
            scrollMain.style.left = `${-w}px`;
            const val = Math.floor(w * factor);
            this.moveWBar.current.style.left = `${val}px`;
        };
        this.barHMove = (e) => {
            const { moveBarH } = this.state;
            const dom = this.moveHBar.current;
            const scrollMain = this.scrollMain.current;
            const startH = +dom.dataset.top;
            const moveDistance = e.clientY - +dom.dataset.y;
            const maxH = this.scrollBoxH - moveBarH;
            let h = startH + moveDistance;
            h = h < 0 ? 0 : h > maxH ? maxH : h;
            const factor = (this.scrollMainH - this.scrollBoxH) / maxH;
            dom.style.top = `${h}px`;
            const top = Math.ceil(-h * factor);
            scrollMain.style.top = `${top}px`;
            const scrollCallback = this.props.scrollCallback;
            scrollCallback && scrollCallback(top, true);
        };
        this.barWMove = (e) => {
            const { moveBarW } = this.state;
            const dom = this.moveWBar.current;
            const scrollMain = this.scrollMain.current;
            const startW = +dom.dataset.left;
            const moveDistance = e.clientX - +dom.dataset.x;
            const maxW = this.scrollBoxW - moveBarW;
            let w = startW + moveDistance;
            w = w < 0 ? 0 : w > maxW ? maxW : w;
            const factor = (this.scrollMainW - this.scrollBoxW) / maxW;
            dom.style.left = `${w}px`;
            const left = Math.ceil(-w * factor);
            scrollMain.style.left = `${left}px`;
            const scrollCallback = this.props.scrollCallback;
            scrollCallback && scrollCallback(left, false);
        };
        this.barHMoveEven = fnUtil.throttle(this.barHMove, 100);
        this.barWMoveEven = fnUtil.throttle(this.barWMove, 100);
        this.upDateInit = (defaultH) => {
            this.updateVertical(defaultH);
            this.updatehorizontal();
        };
        this.barHClick = (e) => {
            e.currentTarget.dataset.y = `${e.clientY}`;
            const top = parseInt(e.currentTarget.style.top || "0", 0);
            e.currentTarget.dataset.top = `${top}`;
            e.currentTarget.parentElement.parentElement.addEventListener("mousemove", this.barHMoveEven);
            this.scrollMain.current.classList.add("no-selected");
        };
        this.barWClick = (e) => {
            e.currentTarget.dataset.x = `${e.clientX}`;
            const left = parseInt(e.currentTarget.style.left || "0", 0);
            e.currentTarget.dataset.left = `${left}`;
            e.currentTarget.parentElement.parentElement.addEventListener("mousemove", this.barWMoveEven);
            this.scrollMain.current.classList.add("no-selected");
        };
        this.cancelBarMove = () => {
            this.moveHBar.current.parentElement.parentElement.removeEventListener("mousemove", this.barHMoveEven);
            this.moveWBar.current.parentElement.parentElement.removeEventListener("mousemove", this.barWMoveEven);
            this.scrollMain.current.classList.remove("no-selected");
        };
        this.mouseOver = (e) => {
            const dom = e.currentTarget;
            if (!dom.firstElementChild) {
                return;
            }
            const touch = e.changedTouches;
            this.startY = touch ? touch[0].pageY : null;
            if (this.state.showWBar) {
                const curScrollBoxW = this.scrollMain.current.parentElement.clientWidth;
                if (curScrollBoxW !== this.scrollBoxW) {
                    this.updatehorizontal();
                }
            }
            if (this.scrollMainH > this.scrollBoxH) {
                dom.onwheel = this.stopWhell;
                dom.ontouchmove = this.stopWhell;
                this.setState({
                    showHBar: true,
                });
                this.moveStartTop = parseInt(this.moveHBar.current.style.top || "0", 0);
            }
            else {
                dom.onwheel = null;
                dom.ontouchmove = null;
            }
        };
        this.mainMousewheel = (e) => {
            this.cancelBarMove();
            const { showHBar, moveBarH } = this.state;
            if (!showHBar) {
                return;
            }
            const touch = e.changedTouches;
            const TouchEndY = touch ? touch[0].pageY : null;
            const isDown = TouchEndY === null ? e.deltaY : TouchEndY - this.startY;
            this.startY = TouchEndY;
            const dom = e.currentTarget;
            const scrollMain = this.scrollMain.current;
            const startH = parseInt(scrollMain.style.top || "0", 0);
            if (this.moveStartTop === null) {
                dom.onwheel = this.stopWhell;
                dom.ontouchmove = this.stopWhell;
            }
            else {
                if (isDown < 0) {
                    if (this.moveStartTop < 5) {
                        dom.onwheel = null;
                        dom.ontouchmove = null;
                    }
                    else {
                        dom.onwheel = this.stopWhell;
                        dom.ontouchmove = this.stopWhell;
                        this.moveStartTop = null;
                    }
                }
                else {
                    if (this.moveStartTop > this.scrollBoxH - moveBarH - 5) {
                        dom.onwheel = null;
                        dom.ontouchmove = null;
                    }
                    else {
                        dom.onwheel = this.stopWhell;
                        dom.ontouchmove = this.stopWhell;
                        this.moveStartTop = null;
                    }
                }
            }
            let step = this.startY ? 10 : Math.sqrt(this.scrollMainH / this.scrollBoxH) * 20;
            step = Math.floor(step);
            step = isDown < 0 ? step : -step;
            let h = -(startH + step);
            const maxH = this.scrollMainH - this.scrollBoxH;
            h = h < 0 ? 0 : h > maxH ? maxH : h;
            const factor = (this.scrollBoxH - moveBarH) / maxH;
            scrollMain.style.top = `${-h}px`;
            this.moveHBar.current.style.top = `${Math.floor(h * factor)}px`;
            const scrollCallback = this.props.scrollCallback;
            scrollCallback && scrollCallback(-h, true);
        };
        this.locateBarH = (e) => {
            const { moveBarH } = this.state;
            let h = e.nativeEvent.offsetY;
            const dom = this.moveHBar.current;
            const maxH = this.scrollBoxH - moveBarH;
            const factor = (this.scrollMainH - this.scrollBoxH) / maxH;
            const scrollMain = this.scrollMain.current;
            h = h < 0 ? 0 : h > maxH ? maxH : h;
            dom.style.top = `${h}px`;
            const top = Math.ceil(-h * factor);
            scrollMain.style.top = `${top}px`;
            const scrollCallback = this.props.scrollCallback;
            scrollCallback && scrollCallback(top, true);
        };
        this.locateBarW = (e) => {
            const { moveBarW } = this.state;
            let w = e.nativeEvent.offsetX;
            const dom = this.moveWBar.current;
            const maxW = this.scrollBoxW - moveBarW;
            const factor = (this.scrollMainW - this.scrollBoxW) / maxW;
            const scrollMain = this.scrollMain.current;
            w = w < 0 ? 0 : w > maxW ? maxW : w;
            dom.style.left = `${w}px`;
            const left = Math.ceil(-w * factor);
            scrollMain.style.left = `${left}px`;
            const scrollCallback = this.props.scrollCallback;
            scrollCallback && scrollCallback(left, false);
        };
        this.stopProp = (e) => {
            e.stopPropagation();
        };
        this.stopWhell = (e) => {
            e.preventDefault();
        };
        this.scrollMethods = {
            initScroll: this.upDateInit,
            scrollToTop: this.scrollToTop,
            scrollToLeft: this.scrollToLeft,
        };
        if (props.bindIntiScroll) {
            props.bindIntiScroll(this.scrollMethods);
        }
    }
    componentDidMount() {
        this.upDateInit(this.props.height);
        document.body.addEventListener("mouseup", this.cancelBarMove);
    }
    componentWillUnmount() {
        document.body.removeEventListener("mouseup", this.cancelBarMove);
        this.props.bindIntiScroll && this.props.bindIntiScroll();
        window.cancelAnimationFrame(this.timerH);
        window.cancelAnimationFrame(this.timerW);
        window.cancelAnimationFrame(this.upTimer);
        this.moveHBar.current.parentElement.parentElement.onwheel = null;
        this.moveHBar.current.parentElement.parentElement.ontouchmove = null;
    }
    updateVertical(setH) {
        if (!this.moveHBar.current) {
            return;
        }
        const barInner = this.moveHBar.current;
        const scrollMain = this.scrollMain.current;
        const curScrollMainH = scrollMain.clientHeight;
        const curScrollBoxH = scrollMain.parentElement.clientHeight;
        if (!setH && curScrollMainH === this.scrollMainH && curScrollBoxH === this.scrollBoxH) {
            return;
        }
        this.scrollBoxH = setH ? setH : curScrollBoxH;
        this.scrollMainH = curScrollMainH;
        if (curScrollMainH <= this.scrollBoxH) {
            scrollMain.style.top = "0";
            this.setState({
                showHBar: false,
            });
            return;
        }
        let moveBarH = Math.ceil((this.scrollBoxH / curScrollMainH) * this.scrollBoxH);
        moveBarH = Math.max(16, moveBarH);
        const top = scrollMain.style.top || "0";
        let curScrollMainTop = -parseInt(top, 0);
        if (curScrollMainTop + this.scrollBoxH > curScrollMainH) {
            curScrollMainTop = curScrollMainH - this.scrollBoxH;
            scrollMain.style.top = `${-curScrollMainTop}px`;
        }
        const maxH = this.scrollMainH - this.scrollBoxH;
        const factor = (this.scrollBoxH - moveBarH) / maxH;
        barInner.style.top = `${Math.floor(factor * curScrollMainTop)}px`;
        this.animateScroll({
            start: this.state.moveBarH,
            end: moveBarH,
            isVertical: true
        }, () => {
            this.setState({
                moveBarH,
                showHBar: true,
            });
        });
    }
    updatehorizontal(setW) {
        if (!this.moveWBar.current) {
            return;
        }
        const barInner = this.moveWBar.current;
        const scrollMain = this.scrollMain.current;
        const curScrollMainW = scrollMain.clientWidth;
        const curScrollBoxW = scrollMain.parentElement.clientWidth;
        if (!setW && curScrollMainW === this.scrollMainW && curScrollBoxW === this.scrollBoxW) {
            return;
        }
        this.scrollBoxW = setW ? setW : curScrollBoxW;
        this.scrollMainW = curScrollMainW;
        if (curScrollMainW <= this.scrollBoxW) {
            scrollMain.style.left = "0";
            this.setState({
                showWBar: false,
            });
            return;
        }
        let moveBarW = Math.ceil((this.scrollBoxW / curScrollMainW) * this.scrollBoxW);
        moveBarW = Math.max(16, moveBarW);
        const left = scrollMain.style.left || "0";
        let curScrollMainLeft = -parseInt(left, 0);
        if (curScrollMainLeft + this.scrollBoxW > curScrollMainW) {
            curScrollMainLeft = curScrollMainW - this.scrollBoxW;
            scrollMain.style.left = `${-curScrollMainLeft}px`;
        }
        const maxW = this.scrollMainW - this.scrollBoxW;
        const factor = (this.scrollBoxW - moveBarW) / maxW;
        barInner.style.left = `${Math.floor(factor * curScrollMainLeft)}px`;
        this.animateScroll({
            start: this.state.moveBarW,
            end: moveBarW,
            isVertical: false
        }, () => {
            this.setState({
                moveBarW,
                showWBar: true,
            });
        });
    }
    animateScroll(obj, callback) {
        const { start, end, isVertical } = obj;
        if (isVertical) {
            window.cancelAnimationFrame(this.timerH);
            this.timerH = 0;
        }
        else {
            window.cancelAnimationFrame(this.timerW);
            this.timerW = 0;
        }
        let time = 0;
        const timeEnd = Math.floor(Math.sqrt(Math.abs(end - start)));
        const fn = isVertical ? () => {
            this.timerH = window.requestAnimationFrame(() => {
                const distance = String(tween.easeInCubic(time, start, end, timeEnd));
                this.moveHBar.current.style.height = distance + "px";
                if (time < timeEnd) {
                    fn();
                }
                else {
                    this.timerH = 0;
                    callback();
                }
                time++;
            });
        } : () => {
            this.timerW = window.requestAnimationFrame(() => {
                const distance = String(tween.easeInCubic(time, start, end, timeEnd));
                this.moveWBar.current.style.width = distance + "px";
                if (time < timeEnd) {
                    fn();
                }
                else {
                    this.timerW = 0;
                    callback();
                }
                time++;
            });
        };
        fn();
    }
    componentDidUpdate(prevProps) {
        if (prevProps.height !== this.props.height) {
            this.upDateInit(this.props.height);
        }
    }
    render() {
        const { className, height, children, scrollWidthAuto } = this.props;
        const { showHBar, moveBarH, moveBarW, showWBar } = this.state;
        const style = height ? { height: `${height}px` } : undefined;
        return (React.createElement("div", { className: `g-scrollBox ${className}`, style: style, onMouseEnter: this.mouseOver, onTouchStart: this.mouseOver, onWheel: this.mainMousewheel, onTouchMove: this.mainMousewheel },
            React.createElement("div", { style: { display: showHBar ? "block" : "none", }, className: "m-scrollBar m-scrollHBar", onClick: this.locateBarH },
                React.createElement("div", { ref: this.moveHBar, className: "m-moveBar m-moveHBar", style: { height: `${moveBarH}px`, top: "0", }, onClick: this.stopProp, onMouseDown: this.barHClick })),
            React.createElement("div", { className: "wrap-main" },
                React.createElement("div", { className: "m-scrollMain", style: { width: scrollWidthAuto ? undefined : "100%" }, ref: this.scrollMain }, children)),
            React.createElement("div", { style: { display: showWBar ? "block" : "none", }, className: "m-scrollBar m-scrollWBar", onClick: this.locateBarW },
                React.createElement("div", { ref: this.moveWBar, className: "m-moveBar m-moveWBar", style: { width: `${moveBarW}px`, left: "0", }, onClick: this.stopProp, onMouseDown: this.barWClick }))));
    }
}
ScrollBox.defaultProps = {
    className: "",
};
export default ScrollBox;
